
//------------------------------------------------------------------->
// Flakes_1.Pov (Snowflake Demo).
//
// Free demonstration file by: Paul T. Dawson.
//                             ptdawson@voicenet.com
//
// For the magnificent raytracer, POV-Ray 3.0 (Thank You, POV-Team!).
//
// Shows objects randomly distributed to fit inside a sphere.
//

//------------------------------------------------------------------->
// Standard settings and include files.

    #version 3.0
    global_settings { assumed_gamma 2.2 }

    #include "colors.inc"
    #include "textures.inc"
    #include "golds.inc"
    #include "skies.inc"

    camera { location < 0, 0, -42 > look_at 0 }
    light_source { < -50, 50, -50 > color White }
    light_source { <  50, 50, -50 > color White }

    #declare R = seed (0)

//------------------------------------------------------------------->
// The two diameter settings control where each object will be placed.
// Change these to almost anything. The Inner_Diameter must be less than
// the Outer_Diameter. You can make Inner_Diameter = 0, to put objects
// everywhere inside the sphere. If you make the two diameters equal,
// then every object will be on the surface of the sphere (the same
// exact distance from the center).

    #declare Outer_Diameter = 17
    #declare Inner_Diameter = 7
    #declare Number_Of_Objects = 300

//------------------------------------------------------------------->
// The object to use can be just about anything. In this scene, I used
// a merge of six boxes. Each one is rotated 45 degrees, and then scaled
// to form a diamond shape. Then the boxes are spun around to form
// something resembling a "snowflake".

    #declare Object_To_Use = merge {

        #declare Loop = 1
        #while ( Loop <= 6 )

            difference{
            
                box { < -1.0, -1.0, -1.0 > < 1.0, 1.0, 1.0 > }
                box { < -0.8, -2.0, -0.8 > < 0.8, 2.0, 0.8 > }
                rotate y * 45
                scale < 1 , 0.2, 0.3 >
                translate x * 1.4
                rotate y * ( Loop * 60 )
                
            } // End of Difference.

        #declare Loop = Loop + 1
        #end

        } // End of merge.

//------------------------------------------------------------------->
// Create the big object.

    #declare Shake_The_Things_In_The_Sphere = union {
        
        #declare Loop = 1
        #while ( Loop <= Number_Of_Objects )

        // Figure out a random distance from the center.
            #declare Move_It =
            ((Outer_Diameter-Inner_Diameter)*rand(R)) + Inner_Diameter

        // Scale the object to whatever size looks best.
            object { Object_To_Use scale 0.7

                // Rotate it before you move it.
                    rotate <rand(R)*360,rand(R)*360,rand(R)*360>

                // Now move it out and spin it again.
                    translate x * Move_It
                    rotate < 0, rand(R)*360, rand(R)*360 >

            } // End of object.

        #declare Loop = Loop + 1 #end

        texture { T_Gold_1E }
        
        } // End of union.

//------------------------------------------------------------------->
// Now, display everything. I rotated the sky 90 degrees, which gives
// a nice "frame" effect with white clouds all around the edges.

    sky_sphere { S_Cloud2 rotate x * 90 }

    object { Shake_The_Things_In_The_Sphere }

//------------------------------------------------------------------->
// End of Flakes_1.Pov - whew!

